using System;
using System.Collections.Generic;
using System.Text;


namespace DataStructures
{

    public delegate void NodeWalkedInOrder(BinaryNode node, BinaryNode parent);         // declaration
    public delegate void NodeWalked(BinaryNode node, BinaryNode parent);                // declaration
        
    public class BinaryNode
    {
       
        public int value;
        public int depth;
        public int offset;
        public BinaryNode left;
        public BinaryNode right;
        public BinaryNode(int _value)
        {
            value = _value;
        }
    }
   
    public class BinaryTree
    {
        public string nodes = ""; 
        public BinaryNode center;
        public NodeWalkedInOrder onNodeWalkedInOrder;
        public NodeWalked onNodeWalked;
                
        public string WalkInOrder()
        {
            string w = DoWalkInOrder(center, true, null);
            return w;
        }

        private string DoWalkInOrder(BinaryNode node, bool useDelegates, BinaryNode parent)
        {
            string retVal = "";
            bool nodeWalked = false;

            if (node == null)
            {
                return "";
            }

            if (useDelegates && onNodeWalked != null) onNodeWalked(node, parent);

            if (node.left == null && node.right == null)
            {
                // it's a leaf
                if (useDelegates && onNodeWalkedInOrder != null) onNodeWalkedInOrder(node, parent);
                retVal += node.value.ToString() + ", ";
            }

            if (node.left != null)
            {
                retVal += DoWalkInOrder(node.left, useDelegates, node);        // find lower values
                retVal += node.value.ToString() + ", ";                 // show yourself
                if (useDelegates && onNodeWalkedInOrder != null) onNodeWalkedInOrder(node, parent);
                nodeWalked = true;
            }
            
            if (node.right != null)
            {
                if (!nodeWalked)
                {
                    if (useDelegates && onNodeWalkedInOrder != null) onNodeWalkedInOrder(node, parent);
                    retVal += node.value.ToString() + ", ";                 // show yourself (if not shown before)
                }
                retVal += DoWalkInOrder(node.right, useDelegates, node);    // find higher values                
            }
            
            return retVal;
        }

        public string ToStringUnordered()
        {
            return nodes;
        }

        public string ToStringOrdered()
        {
            return DoWalkInOrder(center, false, null);
        }

        public void Add(int value)
        {
            nodes += value.ToString() + ", ";
            if (center == null)
            {
                center = new BinaryNode(value);
                center.depth = 1;
                center.offset = 0;
            }
            else 
            {
                BinaryNode pointer = center;
                BinaryNode next;
                int nodeDepth = center.depth;
                int nodeOffset = center.offset;
                while (true)
                {
                    nodeDepth++;
                    if (value <= pointer.value)
                    {
                        nodeOffset--;
                        next = pointer.left;
                        if (next == null)
                        {
                            pointer.left = new BinaryNode(value);
                            pointer.left.depth = nodeDepth;
                            pointer.left.offset = nodeOffset;
                            break;
                        }
                    }
                    else 
                    {
                        nodeOffset++;
                        next = pointer.right;
                        if (next == null)
                        {
                            pointer.right = new BinaryNode(value);
                            pointer.right.depth = nodeDepth;
                            pointer.right.offset = nodeOffset;
                            break;
                        }
                    }
                    pointer = next;
                }
                
            }
        }

    }
}
