using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace DataStructures
{
    public class BinaryTreeDrawer
    {

        public int nodeWidth = 20;
        public int nodeLine = 10;
        public int nodeDepth = 35;

        Graphics g;
        Size nodeSize, maxImageSize, minImageSize;
        Point initPos;

        public void Draw(BinaryTree binaryTree, Graphics g, ref Point _initPos, out Size imageSize)
        {
            this.g = g;
            maxImageSize = new Size(0,0);
            minImageSize = new Size(0,0);

            nodeSize = new Size(nodeWidth, nodeWidth);            
            initPos = _initPos;
            Draw(binaryTree.center, initPos, 'C');

            int width = maxImageSize.Width + Math.Abs(minImageSize.Width)+100;
            int height = maxImageSize.Height + Math.Abs(maxImageSize.Height) + 100;
            imageSize = new Size(width, height);
            _initPos = new Point(Math.Abs(minImageSize.Width)+100, 25);

        }

        private void Draw(BinaryNode node, Point lastPos, char side)        
        {
            if (side == 'C')
            {
                if (node != null)
                {
                    DrawNode(lastPos, nodeSize, node);
                    if (node.left != null) Draw(node.left, lastPos, 'L');
                    if (node.right != null) Draw(node.right, lastPos, 'R');
                }
            }
            else
            {
                int count = 0;
                Point newPos = new Point(lastPos.X, lastPos.Y);
                newPos.Y = node.depth * nodeDepth;

                if (side=='L')
                {
                    count = 1 + WalkNode(node.right);
                    newPos.X -= (count * 2) * nodeLine;
                }
                else
                {
                    count = 1 + WalkNode(node.left);
                    newPos.X += (count * 2) * nodeLine;
                }

                if (newPos.X > maxImageSize.Width) maxImageSize.Width = newPos.X;
                if (newPos.Y > maxImageSize.Height) maxImageSize.Height = newPos.Y;

                if (newPos.X < minImageSize.Width) minImageSize.Width = newPos.X;
                if (newPos.Y < minImageSize.Height) minImageSize.Height = newPos.Y;

                Pen pen = new Pen(Brushes.Blue);
                Point p1 = new Point(lastPos.X + (nodeSize.Width / 2), lastPos.Y + nodeSize.Height);
                Point p2 = new Point(newPos.X + (nodeSize.Width / 2), newPos.Y);
                g.DrawLine(pen, p1, p2);

                DrawNode(newPos, nodeSize, node);
                if (node.left != null) Draw(node.left, newPos, 'L');
                if (node.right != null) Draw(node.right, newPos, 'R');
                
            }

            return;
        }

        public int WalkNode(BinaryNode node)
        {
            int count = 0;
            if (node != null)
            {
                count = 1;
                if (node.left != null) count += WalkNode(node.left);
                if (node.right != null) count += WalkNode(node.right);
            }
            return count;
        }

        public void FindLeftCount(BinaryNode node, ref int rCount, ref int lCount)
        {
            if (node == null)
            {
                return;
            }

            if (node.left != null)
            {
                lCount++;
                FindLeftCount(node.left, ref rCount, ref lCount);
            }

            if (node.right != null)
            {
                FindLeftCount(node.right, ref lCount, ref rCount);
            }
            return;
        }

        public void FindRightCount(BinaryNode node, ref int rCount, ref int lCount)
        {
            if (node == null)
            {
                return;
            }

            if (node.right != null)
            {
                rCount++;
                FindRightCount(node.right, ref rCount, ref lCount);
            }

            if (node.left != null)
            {
                int rCount2 = 0;
                FindRightCount(node.left, ref rCount2, ref lCount);
                if (rCount2 > rCount ) 
                    rCount = rCount2;
            }
        }

        void DrawNode(Point pos, Size size, BinaryNode node)
        {
            // inner circle
            Size fillSize = new Size(size.Width, size.Height);
            Point fillPos = new Point(pos.X + 2, pos.Y + 2);
            Rectangle fillRect = new Rectangle(fillPos, fillSize);
            g.FillEllipse(Brushes.LightGreen, fillRect);

            // outer circle
            Pen pen = new Pen(Brushes.Black);
            Rectangle outerRect = new Rectangle(pos, size);
            g.DrawEllipse(pen, outerRect);

            try
            {
                // text
                PointF textPos = new PointF(pos.X + 2, pos.Y + 5);
                //string text = string.Format("{0} ({1},{2}) {3}", node.value, textPos.X, textPos.Y, node.offset);
                string text = string.Format("{0}", node.value);
                Font f = new Font(FontFamily.GenericSerif, 11);
                g.DrawString(text, f, Brushes.Black, textPos);
                f.Dispose();                
            }
            catch (Exception ex)
            {
                System.Windows.Forms.MessageBox.Show("ex:" + ex.Message);
            }

        }
    }
}


//private int FindMaxOffset(BinaryNode node)
//{
//    int max = node.offset;
//    if (node.right != null)
//    {
//        max = FindMaxOffset(node.right);
//        if (node.left != null)
//        {
//            int max2 = FindMaxOffset(node.left);
//            if (max2 > max) max = max2;
//        }
//    }
//    return max;
//}

//private int FindMinOffset(BinaryNode node)
//{
//    int min = node.offset;
//    if (node.left != null)
//    {
//        min = FindMinOffset(node.left);
//        if (node.right != null)
//        {
//            int min2 = FindMinOffset(node.right);
//            if (min2 < min) min = min2;
//        }
//    }
//    return min;
//}

//private void Draw2(BinaryNode node, Point lastPos, char side)
//{
//    if (side == 'C')
//    {
//        DrawNode(lastPos, nodeSize, node);
//        Draw(node.left, lastPos, 'L');
//        Draw(node.right, lastPos, 'R');
//    }
//    else
//    {
//        int max, min, xinc;
//        Point newPos = new Point(lastPos.X, lastPos.Y);
//        newPos.Y = node.depth * 35;

//        if (side == 'L')
//        {
//            max = FindMaxOffset(node);
//            xinc = 40;
//            newPos.X += -xinc;
//        }
//        else
//        {
//            min = FindMinOffset(node);
//            xinc = (min - 1) * 40;
//            newPos.X += xinc;
//        }

//        DrawNode(newPos, nodeSize, node);
//        if (node.left != null) Draw(node.left, newPos, side);
//        if (node.right != null) Draw(node.right, newPos, side);

//    }

//    return;
//}

//private void Draw1(BinaryNode node, Point lastPos, char side)
//{
//    if (side == 'C')
//    {
//        DrawNode(lastPos, nodeSize, node);
//        Draw(node.left, lastPos, 'L');
//        Draw(node.right, lastPos, 'R');
//    }
//    else
//    {
//        Point newPos = new Point(lastPos.X, lastPos.Y);
//        int def_xinc = 40;
//        int xinc = 0;
//        newPos.Y = node.depth * 35;

//        if (side == 'L')
//        {
//            xinc = def_xinc;
//            if (node.left != null)
//            {
//                xinc += FindRightCount(node) * 40;
//            }
//            if (node.right != null)
//            {
//                xinc += FindRightCount(node) * 40;
//            }
//            newPos.X -= xinc;   // negative since we are on the left side
//        }
//        else
//        {
//            xinc = def_xinc;
//            if (node.left != null)
//            {
//                xinc += FindLeftCount(node) * 40;
//            }
//            if (node.right != null)
//            {
//                xinc += FindLeftCount(node) * 40;
//            }
//            newPos.X += xinc;
//        }

//        DrawNode(newPos, nodeSize, node);
//        if (node.left != null) Draw(node.left, newPos, 'L');
//        if (node.right != null) Draw(node.right, newPos, 'R');

//    }

//    return;
//}

//void xOnNodeWalked(BinaryNode node, BinaryNode parent)
//{
//    sequence++;
//    int y, x;
//    if (sequence == 1)
//    {
//        lastPos = initPos;
//        y = initPos.Y;
//        x = initPos.X;
//    }
//    else
//    {
//        if (parent.value >= node.value)
//        {
//            y = lastPos.Y + 30;
//            x = lastPos.X - 30;
//        }
//        else
//        {
//            y = lastPos.Y + 30;
//            x = lastPos.X + 30;
//        }
//    }
//    sequence++;
//    lastPos.Y = y;
//    lastPos.X = x;

//    Size size = new Size(25, 25);
//    DrawNode(new Point(x, y), size, node);
//}

//void yOnNodeWalked(BinaryNode node, BinaryNode parent)
//{
//    sequence++;
//    int y, x;
//    if (sequence == 1)
//    {
//        y = initPos.Y;
//        x = initPos.X;
//    }
//    else
//    {
//        int factor = ((parent.value > node.value) ? -1 : 1);
//        int r = FindRightCount(node);
//        int l = FindLeftCount(node);
//        y = initPos.Y + ((node.depth - 1) * 30);
//        x = initPos.X;
//        x = initPos.X + (l * factor * 20);
//        x = initPos.X + (r * factor * 20);
//    }
//    sequence++;

//    Size size = new Size(25, 25);
//    //int yOffset = 0;
//    //if (sequence > 1)
//    //{
//    //    yOffset = ((node.depth % 2) == 0) ? 20 : 100;
//    //}
//    ////int y = initPos.Y + ((node.depth-1) * 50) + yOffset;
//    ////int x = initPos.X + (node.offset * ((11-node.depth)*20));

//    //int y = initPos.Y + ((node.depth - 1) * 40);
//    //int x = initPos.X + (node.offset * 50);
//    //if (sequence > 1)
//    //{
//    //    x -= ((node.offset < 0) ? 100 : 0);
//    //}
//    DrawNode(new Point(x,y), size, node);
//}

//void ByMerePositionOnNodeWalked(BinaryNode node, BinaryNode parent)
//{
//    sequence++;
//    int y, x;
//    y = initPos.Y + ((node.depth - 1) * 30);
//    x = (node.value * 15);
//    sequence++;

//    Size size = new Size(25, 25);
//    DrawNode(new Point(x, y), size, node);
//}

//void OnNodeWalked(BinaryNode node, BinaryNode parent)
//{
//    int y, x;
//    sequence++;
//    if (sequence == 1)
//    {
//        lastPos = initPos;
//        y = initPos.Y;
//        x = initPos.X;
//    }
//    else
//    {
//        int l = FindLeftCount(node);
//        int r = FindRightCount(node);
//        if (node.value<10)
//        {
//            if (node.value > parent.value)
//            {
//                y = initPos.Y + (node.depth * 25);
//                x = initPos.X - ((r + l) * 25);
//            }
//            else 
//            {
//                y = initPos.Y + (node.depth * 25);
//                x = initPos.X - ((r + l) * 25);
//            }
//        }
//        else
//        {
//            y = initPos.Y + (node.depth * 25);
//            x = initPos.X + ((r + l) * 25);
//        }
//    }
//    sequence++;
//    lastPos.Y = y;
//    lastPos.X = x;

//    Size size = new Size(25, 25);
//    DrawNode(new Point(x, y), size, node);
//}


