using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;

namespace DataStructures
{
    [TestFixture]
    public class BinaryTreeTest
    {
        private string onWalk = "";

        [Test]
        public void TestSimpleBinaryTree()
        {
            BinaryTree b = new BinaryTree();
            b.Add(10);
            b.Add(5);
            b.Add(15);
            Assert.AreEqual("10, 5, 15, ", b.nodes);
            Assert.AreEqual("5, 10, 15, ", b.WalkInOrder());
        }

        private BinaryTree SetUpLeftChargedTree()
        {
            BinaryTree b = new BinaryTree();
            b.Add(96);
            b.Add(21);
            b.Add(65);
            b.Add(55);
            b.Add(23);
            b.Add(91);
            b.Add(60);
            b.Add(15);
            b.Add(10);
            b.Add(50);
            b.Add(10);
            b.Add(3);
            b.Add(10);
            b.Add(1);
            b.Add(40);
            b.Add(10);
            return b;
        }

        [Test]
        public void TestWalkLeftLoadedTree()
        {
            BinaryTree b = SetUpLeftChargedTree();
            Assert.AreEqual("96, 21, 65, 55, 23, 91, 60, 15, 10, 50, 10, 3, 10, 1, 40, 10, ", b.nodes);
            Assert.AreEqual("1, 3, 10, 10, 10, 10, 15, 21, 23, 40, 50, 55, 60, 65, 91, 96, ", b.WalkInOrder());
        }

        [Test]
        public void TestOnWalkLeftLoadedTree()
        {
            BinaryTree b = SetUpLeftChargedTree();
            b.onNodeWalkedInOrder += new NodeWalkedInOrder(OnWalkLeftLoadedTree);
            Assert.AreEqual("96, 21, 65, 55, 23, 91, 60, 15, 10, 50, 10, 3, 10, 1, 40, 10, ", b.nodes);
            onWalk = "";
            b.WalkInOrder();
            Assert.AreEqual("1, 3, 10, 10, 10, 10, 15, 21, 23, 40, 50, 55, 60, 65, 91, 96, ", onWalk);
        }

        private void OnWalkLeftLoadedTree(BinaryNode node, BinaryNode parent)
        {
            onWalk += node.value.ToString() + ", ";
        }
        
        [Test]
        public void TestToString()
        {
            BinaryTree b = SetUpLeftChargedTree();
            b.onNodeWalkedInOrder += new NodeWalkedInOrder(OnWalkLeftLoadedTree);
            Assert.AreEqual("96, 21, 65, 55, 23, 91, 60, 15, 10, 50, 10, 3, 10, 1, 40, 10, ", b.nodes);
            Assert.AreEqual("1, 3, 10, 10, 10, 10, 15, 21, 23, 40, 50, 55, 60, 65, 91, 96, ", b.ToString());
        }

    }
}
