using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DataStructures;

namespace DataStructuresDemo
{
    public partial class BinaryTreeDemo : Form
    {
        BinaryTree binaryTree;
        
        public BinaryTreeDemo()
        {
            InitializeComponent();
        }

        private void BinaryTreeForm_Load(object sender, EventArgs e)
        {
            Sample2();
        }


        private void cmdAdd_Click(object sender, EventArgs e)
        {
            try
            {
                AddIt(Convert.ToInt32(txtValue.Text));
            }
            catch
            {
                MessageBox.Show("Invalid value entered. Please enter a number.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }
            txtValue.Text = "";
            txtValue.Focus();
        }

        private void cmdRandom_Click(object sender, EventArgs e)
        {
            try
            {
                Random r = new Random();
                int howMany = Convert.ToInt32(txtRandom.Text);
                for (int i = 0; i < howMany; i++)
                {
                    AddIt(r.Next(1000));
                }
                //GraphIt();
            }
            catch (FormatException)
            {
                MessageBox.Show("Invalid value entered. Please enter a positive number.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);            
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }
            txtRandom.Text = "";
            txtRandom.Focus();

        }

        private void AddIt(int value)
        {
            binaryTree.Add(value);
            listUnsorted.Items.Add(value);
            lblStatus.Text = string.Format("Total Elements:{0}", listUnsorted.Items.Count);
        }
        
        private void OnNodeWalkedInOrder(BinaryNode node, BinaryNode parent)
        {
            listSorted.Items.Add(node.value);
        }
        
        private void ResetTree()
        {
            binaryTree = new BinaryTree();
            binaryTree.onNodeWalkedInOrder = new NodeWalkedInOrder(OnNodeWalkedInOrder);
            listUnsorted.Items.Clear();
            listSorted.Items.Clear();
            lblStatus.Text = string.Format("Total Elements:{0}", listUnsorted.Items.Count);
        }

        private void GraphIt()
        {
            GraphForm gf = new GraphForm();
            gf.tree = binaryTree;
            gf.Show();
        }

        /* 
         * Samples 
         */
        private void Sample1()
        {
            ResetTree();
            AddIt(100);
            AddIt(50);
            AddIt(180);
            AddIt(40);
            AddIt(60);
            AddIt(61);
            AddIt(62);
            AddIt(63);
            AddIt(150);
            AddIt(200);
            //GraphIt();
        }

        private void Sample2()
        {
            ResetTree();
            AddIt(10);
            AddIt(1);
            AddIt(20);
            AddIt(19);
            AddIt(18);
            AddIt(17);
            AddIt(16);
            AddIt(15);
            AddIt(14);
            AddIt(13);
            AddIt(12);
            AddIt(11);
            AddIt(14);
            AddIt(13);
            AddIt(12);
            AddIt(11);
            AddIt(2);
            AddIt(3);
            AddIt(4);
            AddIt(9);
            AddIt(6);
            AddIt(7);
            AddIt(2);
            //GraphIt();
        }

        private void Sample3()
        {
            ResetTree();
            AddIt(100);
            AddIt(50);
            AddIt(150);
            AddIt(25);
            AddIt(75);
            AddIt(12);
            AddIt(40);
            AddIt(60);
            AddIt(90);
            //GraphIt();
        }

        private void Sample4()
        {
            ResetTree();
            AddIt(50);
            AddIt(25);
            AddIt(75);
            AddIt(40);
            AddIt(60);
        }

 
        /* 
         * Toolbar and menu 
         */
        private void cmdClear_Click(object sender, EventArgs e)
        {
            ResetTree();
        }

        private void cmdSortX_Click(object sender, EventArgs e)
        {
            listSorted.Items.Clear();
            binaryTree.WalkInOrder();
        }

        private void cmdGraphX_Click(object sender, EventArgs e)
        {
            GraphIt();
        }
        
        private void mnuFileSample1_Click(object sender, EventArgs e)
        {
            Sample1();
        }

        private void mnuFileSample2_Click(object sender, EventArgs e)
        {
            Sample2();
        }

        private void mnuFileSample3_Click(object sender, EventArgs e)
        {
            Sample3();
        }

        private void mnuFileSample4_Click(object sender, EventArgs e)
        {
            Sample4();
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void mnuEditClear_Click(object sender, EventArgs e)
        {
            ResetTree();
        }

        private void mnuViewGraph_Click(object sender, EventArgs e)
        {
            GraphIt();
        }

        private void aboutToolStripMenuItem1_Click(object sender, EventArgs e)
        {
            ShowAbout();
        }

        private void ShowAbout()
        {
            AboutForm f = new AboutForm();
            f.ShowDialog();
        }
    }
}