using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using DataStructures;

namespace DataStructuresDemo
{
    public partial class GraphForm : Form
    {
        public BinaryTree tree;
        bool needRedraw = false;
        private string imageFile;

        public GraphForm()
        {
            InitializeComponent();
        }

        private void GraphForm_Load(object sender, EventArgs e)
        {
            pictureBox1.Left = 0;
            pictureBox1.Top = 0;
            needRedraw = true;
            this.Invalidate(true);
        }

        private void GraphForm_Paint(object sender, PaintEventArgs e)
        {

            if (needRedraw)
            {                
                BinaryTreeDrawer drawer = new BinaryTreeDrawer();
                Graphics g0 = this.CreateGraphics();

                // first shot - just to get the size of the image
                Image i = new Bitmap(100,100, g0);
                Graphics g = Graphics.FromImage(i);
                Size size;
                Point pos = new Point(50, 25);
                drawer.Draw(tree, g, ref pos, out size);
 
                // second shot - draw it for real
                Image i2 = new Bitmap(size.Width, size.Height, g0);
                Graphics g2 = Graphics.FromImage(i2);
                Rectangle rect = new Rectangle(new Point(0, 0), size);
                g2.FillRectangle(Brushes.White, rect);
                drawer.Draw(tree, g2, ref pos, out size);
                needRedraw = false;
                imageFile = NextFileName(Application.StartupPath + @"\binarytree_", ".png");
                i2.Save(imageFile);
                pictureBox1.Image = i2;
                pictureBox1.Width = pictureBox1.Image.Width;
                pictureBox1.Height = pictureBox1.Image.Height;

            }
        }

        public void Redraw()
        {
            needRedraw = true;
            this.Invalidate(true);
        }

        private string NextFileName(string baseName, string ext)
        {
            int sequence = 0;
            string nextFileName;
            while(true)
            {
                sequence++;
                nextFileName = baseName + sequence.ToString() + ext;
                if (!System.IO.File.Exists(nextFileName))
                {
                    break;
                }
            }
            return nextFileName;
        }

        private void NormalGraph()
        {
            pictureBox1.SizeMode = PictureBoxSizeMode.Normal;
            pictureBox1.Width = pictureBox1.Image.Width;
            pictureBox1.Height = pictureBox1.Image.Height; 
            Redraw();
        }

        private void StretchGraph()
        {
            pictureBox1.SizeMode = PictureBoxSizeMode.StretchImage;
            pictureBox1.Width = this.Width;
            pictureBox1.Height = this.Height;
            Redraw();
        }

        private void ZoomGraph()
        {
            pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
            pictureBox1.Width = this.Width;
            pictureBox1.Height = this.Height;
            Redraw();
        }

        private void SaveGraph()
        {
            saveFileDialog1.FileName = "MyGraph";
            saveFileDialog1.Filter = "Portable Network Graphics files (*.png)|*.png";
            saveFileDialog1.DefaultExt = "png";
            saveFileDialog1.Title = "Save Graph As";
            DialogResult r = saveFileDialog1.ShowDialog();
            if (r == DialogResult.OK)
            {
                pictureBox1.Image.Save(saveFileDialog1.FileName);
            }
        }

        private void GraphForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (System.IO.File.Exists(imageFile))
            {
                try
                {
                    System.IO.File.Delete(imageFile);
                }
                catch 
                {   
                    // ignore it 
                }
            }
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void normalToolStripMenuItem_Click(object sender, EventArgs e)
        {
            NormalGraph();
        }

        private void stretchToolStripMenuItem_Click(object sender, EventArgs e)
        {
            StretchGraph();
        }

        private void zoomToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ZoomGraph();
        }

        private void saveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            SaveGraph();
        }
      
    }
}